import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { loginSchema } from '@/models/loginSchema';
import { sessionSchemaValid } from '@/models/sessionValidSchema';
import { Login, SessionValid, UserPrivileges } from '@/types/models';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { USER_ROLE } from '@/constants/user';
import { apiPath } from '../apiPath';
import { closeModal, openLoggedInMenuModal } from '../modal/modal.slice';
import { hasPrivilege } from './user.utils';

const getUserRole = async (login: string): Promise<string> => {
  const response = await axiosInstance.get<UserPrivileges>(apiPath.userPrivileges(login), {
    withCredentials: true,
  });

  const {
    data: { privileges },
  } = response;

  if (hasPrivilege(privileges, 'IS_ADMIN')) {
    return USER_ROLE.ADMIN;
  }
  if (hasPrivilege(privileges, 'IS_CURATOR')) {
    return USER_ROLE.CURATOR;
  }

  return USER_ROLE.USER;
};

export const login = createAsyncThunk(
  'user/login',
  async (credentials: { login: string; password: string }, { dispatch, rejectWithValue }) => {
    try {
      const searchParams = new URLSearchParams(credentials);
      const response = await axiosInstance.post<Login>(apiPath.postLogin(), searchParams, {
        withCredentials: true,
      });

      const isDataValid = validateDataUsingZodSchema(response.data, loginSchema);
      const loginName = response.data.login;

      if (isDataValid) {
        const role = await getUserRole(loginName);

        if (role !== USER_ROLE.ADMIN && role !== USER_ROLE.CURATOR) {
          dispatch(closeModal());
        } else {
          dispatch(openLoggedInMenuModal());
        }

        return {
          login: loginName,
          role,
        };
      }

      return undefined;
    } catch (error) {
      const errorMessage = getErrorMessage({
        error,
        prefix: 'Login',
      });

      return rejectWithValue(errorMessage);
    }
  },
);

export const getSessionValid = createAsyncThunk('user/getSessionValid', async () => {
  const response = await axiosInstance.get<SessionValid>(apiPath.getSessionValid(), {
    withCredentials: true,
  });

  const isDataValid = validateDataUsingZodSchema(response.data, sessionSchemaValid);

  const {
    data: { login: loginName },
  } = response;

  const role = await getUserRole(loginName);

  if (isDataValid) {
    return {
      login: loginName,
      role,
    };
  }

  return undefined;
});

export const logout = createAsyncThunk('user/logout', async (_, { rejectWithValue }) => {
  try {
    await axiosInstance.post(apiPath.logout(), null, {
      withCredentials: true,
    });

    return undefined;
  } catch (error) {
    const errorMessage = getErrorMessage({
      error,
      prefix: 'Log out',
    });

    return rejectWithValue(errorMessage);
  }
});
